GSE_DATABASE_FRAME_LIST_BUTTON_HEIGHT = 16;
GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE = 50;
GSE_DATABASE_FRAME_LIST_VISIBLE_RESULTS = 20;
GSE_DATABASE_FRAME_COLUMNS = 10;

function GSE_DatabaseFrame_OnLoad(self)
	tinsert(UISpecialFrames, self:GetName());
	
	self:RegisterForDrag("LeftButton");

	self.TitleText:SetText("Player Database");

	SetPortraitToTexture(GSE_DatabaseFramePortrait, "Interface\\Spellbook\\Spellbook-Icon");
	
	self.page = 1;
	self.searchResults = {};
	self.sortAsc = true;
	self.sortColumn = 1;
	
	GSE_DatabaseFrame_UpdateSearchResults(self);
	GSE_DatabaseFrame_UpdateSortIcons(self);
end

function GSE_DatabaseFrame_UpdateSortIcons(self)
	for i = 1, GSE_DATABASE_FRAME_COLUMNS do
		local buttonName = self:GetName().."ColumnHeader"..i;
		local arrow = _G[buttonName.."Arrow"];
		if i == self.sortColumn then
			if self.sortAsc then
				arrow:SetTexCoord(0, 0.5625, 1.0, 0);
				arrow:ClearAllPoints();
				arrow:SetPoint("LEFT", _G[buttonName.."Text"], "RIGHT", 3, 1);
			else
				arrow:SetTexCoord(0, 0.5625, 0, 1.0);
				arrow:ClearAllPoints();
				arrow:SetPoint("LEFT", _G[buttonName.."Text"], "RIGHT", 3, 0);
			end
			arrow:Show();
		else
			arrow:Hide();
		end
	end
end

function GSE_DatabaseFrame_UpdateSearchResults(self)
	self.page = 1;
	wipe(self.searchResults);
	if gGSEPlayerData then
		local search = GSE_DatabaseFrameSearchBox:GetText();
		if search then
			search = search:lower();
		end
		for key, playerData in pairs(gGSEPlayerData) do
			local match = true;
			if search then
				local haystack = playerData["name"].." "..playerData["realm"].." "..playerData["level"].." "..playerData["race"].." "..playerData["class"].." "..GSE_GearScore(playerData).." "..GSE_TimeIntervalString(time() - playerData["infoTime"]);
				if playerData["guildName"] then
					haystack = haystack.." "..playerData["guildName"];
				end
				if playerData["talents"] and gGSETalentTreeData[playerData["class"]] then
					for i = 1, 2 do
						if playerData["talents"][i] and playerData["talents"][i]["primaryTab"] then
							haystack = haystack.." "..format("%s %d %d %d", gGSETalentTreeData[playerData["class"]][playerData["talents"][i]["primaryTab"]]["name"], playerData["talents"][i][1]["pointsSpent"], playerData["talents"][i][2]["pointsSpent"], playerData["talents"][i][3]["pointsSpent"]);
						else
							haystack = haystack.." Undecided";
						end
					end
				end
				haystack = haystack:lower();
				local parts = {strsplit(" ", search:lower())};
				for i = 1, #parts do
					if not strfind(haystack, parts[i]) then
						match = false;
						break;
					end
				end
			end
			if match then
				tinsert(self.searchResults, playerData);
			end
		end
	end
	
	GSE_DatabaseFrame_SortSearchResults(self);
end

function GSE_DatabaseFrame_SpecComparator(a, b, spec, asc)
	local stringa = "";
	local stringb = "";
	if a["talents"] and gGSETalentTreeData[a["class"]] then
		if a["talents"][spec] and a["talents"][spec]["primaryTab"] then
			stringa = format("%s %d %d %d", gGSETalentTreeData[a["class"]][a["talents"][spec]["primaryTab"]]["name"], a["talents"][spec][1]["pointsSpent"], a["talents"][spec][2]["pointsSpent"], a["talents"][spec][3]["pointsSpent"]);
		else
			stringa = "Undecided";
		end
	end
	if b["talents"] and gGSETalentTreeData[b["class"]] then
		if b["talents"][spec] and b["talents"][spec]["primaryTab"] then
			stringb = format("%s %d %d %d", gGSETalentTreeData[b["class"]][b["talents"][spec]["primaryTab"]]["name"], b["talents"][spec][1]["pointsSpent"], b["talents"][spec][2]["pointsSpent"], b["talents"][spec][3]["pointsSpent"]);
		else
			stringb = "Undecided";
		end
	end
	if asc then
		return stringa < stringb;
	end
	return stringa > stringb;
end

function GSE_DatabaseFrame_SortSearchResults(self)
	local comparatorsAsc = {
		[1] = function(a, b) return a["name"]:lower() < b["name"]:lower(); end,
		[2] = function(a, b) return a["realm"]:lower() < b["realm"]:lower(); end,
		[3] = function(a, b) return a["level"] < b["level"]; end,
		[4] = function(a, b) return a["race"] < b["race"]; end,
		[5] = function(a, b) return a["class"] < b["class"]; end,
		[6] = function(a, b) return b["guildName"] and (not a["guildName"] or a["guildName"]:lower() < b["guildName"]:lower()); end,
		[7] = function(a, b) return GSE_DatabaseFrame_SpecComparator(a, b, 1, true); end,
		[8] = function(a, b) return GSE_DatabaseFrame_SpecComparator(a, b, 2, true); end,
		[9] = function(a, b) return GSE_GearScore(a) < GSE_GearScore(b); end,
		[10] = function(a, b) return a["infoTime"] > b["infoTime"]; end,
	};

	local comparatorsDesc = {
		[1] = function(a, b) return a["name"]:lower() > b["name"]:lower(); end,
		[2] = function(a, b) return a["realm"]:lower() > b["realm"]:lower(); end,
		[3] = function(a, b) return a["level"] > b["level"]; end,
		[4] = function(a, b) return a["race"] > b["race"]; end,
		[5] = function(a, b) return a["class"] > b["class"]; end,
		[6] = function(a, b) return a["guildName"] and (not b["guildName"] or a["guildName"]:lower() > b["guildName"]:lower()); end,
		[7] = function(a, b) return GSE_DatabaseFrame_SpecComparator(a, b, 1, false); end,
		[8] = function(a, b) return GSE_DatabaseFrame_SpecComparator(a, b, 2, false); end,
		[9] = function(a, b) return GSE_GearScore(a) > GSE_GearScore(b); end,
		[10] = function(a, b) return a["infoTime"] < b["infoTime"]; end,
	};
	
	if self.sortAsc then
		sort(self.searchResults, comparatorsAsc[self.sortColumn]);
	else
		sort(self.searchResults, comparatorsDesc[self.sortColumn]);
	end

	GSE_DatabaseFrameListScrollFrame:SetVerticalScroll(0);
	GSE_DatabaseFrame_List_Update(GSE_DatabaseFrameListScrollFrame);
	GSE_DatabaseFrame_UpdatePageNumber(self);
end

function GSE_DatabaseFrame_UpdatePageNumber(self)
	if self.page > 1 then
		GSE_DatabaseFramePageLeftButton:Enable();
	else
		GSE_DatabaseFramePageLeftButton:Disable();
	end

	local pages = ceil(#self.searchResults / GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE);
	if self.page < pages then
		GSE_DatabaseFramePageRightButton:Enable();
	else
		GSE_DatabaseFramePageRightButton:Disable();
	end
	
	if pages > 0 then
		GSE_DatabaseFramePageNumber:SetFormattedText("Page %d of %d", self.page, pages);
	else
		GSE_DatabaseFramePageNumber:SetText("No results");
	end
end

function GSE_DatabaseFrame_OnShow(self)
	PlaySound("igCharacterInfoOpen");
	GSE_DatabaseFrame_UpdateSearchResults(self);
end

function GSE_DatabaseFrame_OnHide(self)
	PlaySound("igCharacterInfoClose");
end

function GSE_DatabaseFrame_SearchBox_OnTextChanged(self)
	GSE_DatabaseFrame_UpdateSearchResults(self:GetParent());
end

function GSE_DatabaseFrame_List_Update(scrollFrame)
	local dbFrame = GSE_DatabaseFrame;
	if dbFrame.page then
		local resultOffset = (dbFrame.page - 1) * GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE;
		local resultsInPage = #dbFrame.searchResults - resultOffset;
		if resultsInPage < 0 then
			resultsInPage = 0;
		elseif resultsInPage > GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE then
			resultsInPage = GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE;
		end
		local offset = FauxScrollFrame_GetOffset(scrollFrame);
		for i = 1, GSE_DATABASE_FRAME_LIST_RESULTS_PER_PAGE do
			local buttonName = scrollFrame:GetParent():GetName().."ListButton"..i;
			if not _G[buttonName] then
				_G[buttonName] = CreateFrame("Button", buttonName, scrollFrame:GetParent(), "GSE_DatabaseFrameListButtonTemplate");
			end
			local button = _G[buttonName];
			if i <= resultsInPage and i > offset and i <= offset + GSE_DATABASE_FRAME_LIST_VISIBLE_RESULTS then		
				button:ClearAllPoints();
				button:SetPoint("TOPLEFT", scrollFrame, "TOPLEFT", 8, (-i + offset + 1) * GSE_DATABASE_FRAME_LIST_BUTTON_HEIGHT);
				button:SetPoint("BOTTOMRIGHT", scrollFrame, "TOPRIGHT", -1, (-i + offset) * GSE_DATABASE_FRAME_LIST_BUTTON_HEIGHT);
		
				local playerData = dbFrame.searchResults[resultOffset + i];
				
				button.playerData = playerData;

				button.name:SetFormattedText("%s%s", GSE_ClassColorString(playerData["class"]), playerData["name"]); 
				button.realm:SetFormattedText("|cFFFFFFFF%s", playerData["realm"]); 
				button.level:SetFormattedText("|cFFFFFFFF%d", playerData["level"]); 
				button.race:SetFormattedText("|cFFFFFFFF%s", playerData["race"]); 
				button.class:SetFormattedText("%s%s", GSE_ClassColorString(playerData["class"]), playerData["class"]); 
				
				if playerData["guildName"] then
					button.guild:SetFormattedText("|cFFFF8080%s", playerData["guildName"]); 
				else
					button.guild:SetText(""); 
				end

				button.primarySpec:SetText("");
				button.secondarySpec:SetText("");
				
				if playerData["talents"] and gGSETalentTreeData[playerData["class"]] then
					if playerData["talents"][1] and playerData["talents"][1]["primaryTab"] then
						local specColor = "|cFFA0A0A0";
						if playerData["talentGroup"] == 1 then
							specColor = "|cFFFFFFFF";
						end
						button.primarySpec:SetFormattedText("%s%s %d %d %d", specColor, gGSETalentTreeData[playerData["class"]][playerData["talents"][1]["primaryTab"]]["name"], playerData["talents"][1][1]["pointsSpent"], playerData["talents"][1][2]["pointsSpent"], playerData["talents"][1][3]["pointsSpent"]);
					else
						button.primarySpec:SetText("|cFFA0A0A0Undecided");
					end

					if playerData["talents"][2] and playerData["talents"][2]["primaryTab"] then
						local specColor = "|cFFA0A0A0";
						if playerData["talentGroup"] == 2 then
							specColor = "|cFFFFFFFF";
						end
						button.secondarySpec:SetFormattedText("%s%s %d %d %d", specColor, gGSETalentTreeData[playerData["class"]][playerData["talents"][2]["primaryTab"]]["name"], playerData["talents"][2][1]["pointsSpent"], playerData["talents"][2][2]["pointsSpent"], playerData["talents"][2][3]["pointsSpent"]);
					else
						button.secondarySpec:SetText("|cFFA0A0A0Undecided");
					end
				end

				local gearscore = GSE_GearScore(playerData);
				local interval = time() - playerData["infoTime"];
				local gsColor = GSE_LightenedColorString(GSE_GearScoreColor(gearscore));
				local tIColor = GSE_LightenedColorString(GSE_DirectGradientColor(interval, 60 * 60 * 24 * 7));
				button.gearscore:SetFormattedText("%s%s", gsColor, gearscore);
				button.cachetime:SetFormattedText("%s%s", tIColor, GSE_TimeIntervalString(interval));
				
				button:Show();
			else
				button:Hide();
			end
		end
		FauxScrollFrame_Update(scrollFrame, resultsInPage, GSE_DATABASE_FRAME_LIST_VISIBLE_RESULTS, GSE_DATABASE_FRAME_LIST_BUTTON_HEIGHT);
	else
		FauxScrollFrame_Update(scrollFrame, 0, 0, GSE_DATABASE_FRAME_LIST_BUTTON_HEIGHT);
	end
end

function GSE_DatabaseFrame_ListButton_OnClick(self, button)
	if button == "LeftButton" then
		GSE_OpenPlayerData(self.playerData);
	end
end

function GSE_DatabaseFrame_ColumnButton_OnClick(self)
	local dbFrame = self:GetParent();
	if dbFrame.sortColumn == self:GetID() then
		dbFrame.sortAsc = not dbFrame.sortAsc;
	else
		dbFrame.sortColumn = self:GetID();
	end
	GSE_DatabaseFrame_SortSearchResults(dbFrame);
	GSE_DatabaseFrame_UpdateSortIcons(dbFrame);
end

function GSE_DatabaseFrame_PageButton_OnClick(self)
	local dbFrame = GSE_DatabaseFrame;
	if self:GetID() == 1 then
		dbFrame.page = dbFrame.page - 1;
	else
		dbFrame.page = dbFrame.page + 1;
	end
	
	GSE_DatabaseFrameListScrollFrame:SetVerticalScroll(0);
	GSE_DatabaseFrame_List_Update(GSE_DatabaseFrameListScrollFrame);
	GSE_DatabaseFrame_UpdatePageNumber(dbFrame);
end