local equipLocCoefficients = {
	["INVTYPE_HEAD"] = 1.0,
	["INVTYPE_NECK"] = 0.5625,
	["INVTYPE_SHOULDER"] = 0.75,
	["INVTYPE_CHEST"] = 1.0,
	["INVTYPE_ROBE"] = 1.0,
	["INVTYPE_WAIST"] = 0.75,
	["INVTYPE_LEGS"] = 1.0,
	["INVTYPE_FEET"] = 0.75,
	["INVTYPE_WRIST"] = 0.5625,
	["INVTYPE_HAND"] = 0.75,
	["INVTYPE_FINGER"] = 0.5625,
	["INVTYPE_TRINKET"] = 0.5625,
	["INVTYPE_CLOAK"] = 0.5625,
	["INVTYPE_WEAPON"] = 1.0,
	["INVTYPE_SHIELD"] = 1.0,
	["INVTYPE_2HWEAPON"] = 2.0,
	["INVTYPE_WEAPONMAINHAND"] = 1.0,
	["INVTYPE_WEAPONOFFHAND"] = 1.0,
	["INVTYPE_HOLDABLE"] = 1.0,
	["INVTYPE_RANGED"] = 0.3164,
	["INVTYPE_THROWN"] = 0.3164,
	["INVTYPE_RANGEDRIGHT"] = 0.3164,
	["INVTYPE_RELIC"] = 0.3164,
	["INVTYPE_QUIVER"] = 0.0,
};

local rarityCoefficients = {
	[0] = {0.005,   73.0,    1.0,  8.0, 2.0},
	[1] = {0.005,   73.0,    1.0,  8.0, 2.0},
	[2] = {  1.0,   73.0,    1.0,  8.0, 2.0},
	[3] = {  1.0, 81.375, 0.8125, 0.75, 1.8},
	[4] = {  1.0,  91.45,   0.65,   26, 1.2},
	[5] = {  1.3,  91.45,   0.65,   26, 1.2},
	[6] = {  0.0,    0.0,    0.0,  0.0, 0.0},
	[7] = {  1.0,  91.45,   0.65,   26, 1.2},
};

function GSE_GearScore(playerData, talentGroup)
	if not playerData or not playerData["inventory"] then
		return 0;
	end
	if not talentGroup then
		talentGroup = playerData["talentGroup"];
	end
	if not playerData["inventory"][talentGroup] then
		return 0;
	end
	local isTitan = false;
	if playerData["class"] == "WARRIOR" and playerData["inventory"][talentGroup]["SecondaryHandSlot"] then
		local _, _, _, _, _, _, _, _, itemEquipLoc = GetItemInfo(playerData["inventory"][talentGroup]["SecondaryHandSlot"]);
		isTitan = (itemEquipLoc == "INVTYPE_2HWEAPON");
	end
	local gearScore = 0;
	for slot, item in pairs(playerData["inventory"][talentGroup]) do
		local _, _, _, _, _, _, _, _, itemEquipLoc = GetItemInfo(item);
		local itemScore = GSE_ItemGearScore(item);
		if playerData["class"] == "HUNTER" and (itemEquipLoc == "INVTYPE_WEAPON" or itemEquipLoc == "INVTYPE_2HWEAPON" or itemEquipLoc == "INVTYPE_WEAPONMAINHAND" or itemEquipLoc == "INVTYPE_WEAPONOFFHAND") then
			itemScore = itemScore * 0.3164;
		elseif playerData["class"] == "HUNTER" and itemEquipLoc == "INVTYPE_RANGED" then
			itemScore = itemScore * 5.3224;
		end
		if isTitan and (itemEquipLoc == "INVTYPE_WEAPON" or itemEquipLoc == "INVTYPE_2HWEAPON" or itemEquipLoc == "INVTYPE_WEAPONMAINHAND" or itemEquipLoc == "INVTYPE_WEAPONOFFHAND") then
			itemScore = itemScore * 0.5;
		end
		gearScore = gearScore + floor(itemScore);
	end
	return gearScore;
end

function GSE_ItemGearScore(item)
	if not item then
		return 0;
	end
	local _, _, itemRarity, itemLevel, _, _, _, _, itemEquipLoc = GetItemInfo(item);
	if not equipLocCoefficients[itemEquipLoc] then
		return 0;
	end
	if itemRarity == 7 then
		itemLevel = 187.05;
		itemRarity = 3;
	end
	local c = rarityCoefficients[itemRarity];
	local gearScore;
	if itemLevel > 120 then
		gearScore = floor(1.8618 * equipLocCoefficients[itemEquipLoc] * c[1] * (itemLevel - c[2]) / c[3]);
	else
		gearScore = floor(1.8618  * equipLocCoefficients[itemEquipLoc] * c[1] * (itemLevel - c[4]) / c[5]);
	end
	if gearScore < 0 then
		return 0;
	end
	return gearScore;
end

function GSE_GearScoreColor(gearScore)
	return GSE_IndirectGradientColor(gearScore, 6300);
end

function GSE_ItemGearScoreColor(item)
	-- All we take into account here is the item level and rarity.
	local _, _, itemRarity, itemLevel = GetItemInfo(item);
	if itemRarity == 7 then
		itemLevel = 187.05;
		itemRarity = 3;
	end
	local c = rarityCoefficients[itemRarity];
	return GSE_IndirectGradientColor(c[1] * itemLevel, 284);
end

function GSE_PaperDollFrame_SetGearScore(statFrame, unit)
	if (not unit) then
		unit = "player";
	end
	local guid = UnitGUID(unit);
	local realm = GSE_Realm(select(2, UnitName(unit)));
	local playerData = GSE_PlayerData(guid, realm);
	if playerData then
		local gearScore = GSE_GearScore(playerData);
		local gsColor = GSE_LightenedColorString(GSE_GearScoreColor(gearScore));
		PaperDollFrame_SetLabelAndText(statFrame, "GearScore", format("%s%d", gsColor, gearScore), false);
		statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..format(PAPERDOLLFRAME_TOOLTIP_FORMAT, "GearScore").." "..format("%s%d", gsColor, gearScore)..FONT_COLOR_CODE_CLOSE;
		statFrame.tooltip2 = "Your GearScore. It represents the quality of your gear. For tools to better assess a player's gems, enchants, gear choices, and skill, see the detailed information window.";
		statFrame:Show();
	end
end

PAPERDOLL_STATINFO["GSE_GEARSCORE"] = {
	updateFunc = function(statFrame, unit) GSE_PaperDollFrame_SetGearScore(statFrame, unit); end
};
PAPERDOLL_STATCATEGORIES["GENERAL"]["stats"][#PAPERDOLL_STATCATEGORIES["GENERAL"]["stats"] + 1] = "GSE_GEARSCORE";