function GSE_OptionsFrame_Okay(self)
	gGSEOptions["showTooltipItemGS"]     = GSE_OptionsFrameTooltipItemGSButton:GetChecked();
	gGSEOptions["showTooltipPlayerGS"]   = GSE_OptionsFrameTooltipPlayerGSButton:GetChecked();
	gGSEOptions["showTooltipTalents"]    = GSE_OptionsFrameTooltipTalentsButton:GetChecked();
	gGSEOptions["showTooltipExperience"] = GSE_OptionsFrameTooltipExperienceButton:GetChecked();
	gGSEOptions["cacheTime"] = UIDropDownMenu_GetSelectedValue(GSE_OptionsFrameCacheTimeDropDown);
end

function GSE_OptionsFrame_Cancel(self)
	-- Do nothing.
end

-- This is also called by GearScoreElite.lua if gGSEOptions does not exist.
function GSE_OptionsFrame_Default(self)
	gGSEOptions = {
		["showTooltipItemGS"]     = true,  -- The typical user expects this.
		["showTooltipPlayerGS"]   = true,  -- The typical user expects this.
		["showTooltipTalents"]    = true,  -- Talent data is available as soon as GearScore is and is fairly noob-friendly.
		["showTooltipExperience"] = false, -- We'll leave this disabled by default since it's really only relevant to advanced players and pug leaders.
		["cacheTime"]             = 60 * 60 * 24 * 7,
	};
	
	GSE_OptionsFrame_Refresh(self);
end

function GSE_OptionsFrame_Refresh(self)
	GSE_OptionsFrameTitle:SetText("GearScore Elite");
	GSE_OptionsFrameSubTitle:SetText("GearScore Elite is an attempt to improve on the original GearScore addon. It adds additional features (with many more to come) and offers stability that GearScore never could.");
	GSE_OptionsFrameSubTitle:SetWordWrap(true);
	
	GSE_OptionsFrameTooltipItemGSButtonText:SetText("Show GearScore in Item Tooltips");
	GSE_OptionsFrameTooltipItemGSButton.tooltipText = "Show item GearScores in your tooltips.";
	GSE_OptionsFrameTooltipItemGSButton:SetChecked(gGSEOptions["showTooltipItemGS"]);

	GSE_OptionsFrameTooltipPlayerGSButtonText:SetText("Show GearScore in Player Tooltips");
	GSE_OptionsFrameTooltipPlayerGSButton.tooltipText = "Show player GearScores in your tooltips.";
	GSE_OptionsFrameTooltipPlayerGSButton:SetChecked(gGSEOptions["showTooltipPlayerGS"]);
	
	GSE_OptionsFrameTooltipTalentsButtonText:SetText("Show Talents in Tooltips");
	GSE_OptionsFrameTooltipTalentsButton.tooltipText = "Show player main-spec and off-spec summaries in your tooltips.";
	GSE_OptionsFrameTooltipTalentsButton:SetChecked(gGSEOptions["showTooltipTalents"]);
	
	GSE_OptionsFrameTooltipExperienceButtonText:SetText("Show Raid Experience in Tooltips");
	GSE_OptionsFrameTooltipExperienceButton.tooltipText = "Show player raid experience in your tooltips.";
	GSE_OptionsFrameTooltipExperienceButton:SetChecked(gGSEOptions["showTooltipExperience"]);
	
	UIDropDownMenu_Initialize(GSE_OptionsFrameCacheTimeDropDown, GSE_CacheTimeDropDown_Initialize);
	UIDropDownMenu_SetSelectedValue(GSE_OptionsFrameCacheTimeDropDown, gGSEOptions["cacheTime"]);
	GSE_OptionsFrameCacheTimeDropDown.tooltipText = "This is the approximate amount of time that player data is stored in your cache.";

	GSE_OptionsFrameCacheTimeDropDownLabel:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1.0);

	local tips = {
		"You can type /gse to show detailed player information, or you set a keybind for this functionality.",
		"You can type /gse db to view your player database, or you set a keybind for this functionality.",
	};
	GSE_OptionsFrameTip:SetFormattedText("Tip: %s", tips[random(#tips)]);
end

function GSE_OptionsFrame_OnLoad(self)
	self.name = "GearScore Elite";
	
	self.okay = GSE_OptionsFrame_Okay;
	self.cancel = GSE_OptionsFrame_Cancel;
	self.default = GSE_OptionsFrame_Default;
	self.refresh = GSE_OptionsFrame_Refresh;
	
	InterfaceOptions_AddCategory(self);
end

function GSE_CacheTimeDropDown_Initialize()
	local selectedValue = UIDropDownMenu_GetSelectedValue(GSE_OptionsFrameCacheTimeDropDown);
	local info = UIDropDownMenu_CreateInfo();
	
	info.text  = "15 minutes";
	info.value = 15 * 60;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);

	info.text  = "1 hour";
	info.value = 60 * 60;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);

	info.text  = "1 day";
	info.value = 60 * 60 * 24;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);

	info.text  = "1 week";
	info.value = 60 * 60 * 24 * 7;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);

	info.text  = "1 month";
	info.value = 60 * 60 * 24 * 31;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);

	info.text  = "Forever";
	info.value = math.huge;
	info.func  = GSE_CacheTimeDropDown_OnClick;
	if info.value == selectedValue then
		info.checked = 1;
	else
		info.checked = nil;
	end
	UIDropDownMenu_AddButton(info);
end

function GSE_CacheTimeDropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(GSE_OptionsFrameCacheTimeDropDown, self.value);
end
